// Buffer indexing expression (used only during parsing) -*- c++ -*-

#ifdef __GNUC__
# pragma implementation
#endif // __GNUC__
#include "BufferIndex.h"

/** @file BufferIndex.C
 * Buffer indexing expression (transient storage used during parsing)
 */

BufferIndex::BufferIndex (class Expression& buffer,
			  class Expression& i) :
  myBuffer (&buffer), myIndex (&i)
{
  assert (myBuffer && myIndex);
  assert (myBuffer->getType ()->getKind () == Type::tBuffer);
  assert (myIndex->getType ()->getKind () == Type::tCard);
  assert (myBuffer->isBasic () && myIndex->isBasic ());

  setType (*myBuffer->getType ());
}

BufferIndex::~BufferIndex ()
{
  myBuffer->destroy ();
  myIndex->destroy ();
}

class Value*
BufferIndex::do_eval (const class Valuation& valuation) const
{
  return myBuffer->eval (valuation);
}

class Expression*
BufferIndex::ground (const class Valuation& valuation,
		     class Transition* transition,
		     bool declare)
{
  return myBuffer->ground (valuation, transition, declare);
}

class Expression*
BufferIndex::substitute (class Substitution& substitution)
{
  return myBuffer->substitute (substitution);
}

bool
BufferIndex::depends (const class VariableSet& vars,
		      bool complement) const
{
  return myBuffer->depends (vars, complement);
}

bool
BufferIndex::forVariables (bool (*operation)
			   (const class Expression&,void*),
			   void* data) const
{
  return myBuffer->forVariables (operation, data);
}

#ifdef EXPR_COMPILE

void
BufferIndex::compile (class CExpression& cexpr,
		      unsigned indent,
		      const char* lvalue,
		      const class VariableSet* vars) const
{
  myBuffer->compile (cexpr, indent, lvalue, vars);
}

#endif // EXPR_COMPILE

void
BufferIndex::display (const class Printer& printer) const
{
  myBuffer->display (printer);
}
