/* Efficient file abstraction layer -*- c++ -*- */

#ifndef FILE_H_
# define FILE_H_

/** @file file.h
 * Efficient file abstraction layer.
 * Use mmap(2) and friends on Unix systems when USE_MMAP is defined.
 * As a portable fall-back method, use <cstdio> FILE pointers.
 */

# include <stdio.h>

# ifndef unix
#  if defined __unix||defined __unix__||defined __NetBSD__||defined __APPLE__
#   define unix
#  endif
# endif

# if defined unix && defined USE_MMAP
#  include <unistd.h>
#  ifdef NO_MMAP
#   include <stdlib.h>
#  else // NO_MMAP
#   include <sys/mman.h>
#  endif // NO_MMAP
/** File handle */
typedef struct
{
#  ifndef NO_MMAP
  int fd;	/**< UNIX file descriptor */
#  endif // !NO_MMAP
  long len;	/**< used file length in bytes */
  long alloc;	/**< allocated file length in bytes */
#  ifdef __sun
  caddr_t addr;	/**< address of the mapped memory area */
#  else // __sun
  void* addr;	/**< address of the mapped memory area */
#  endif // __sun
} file_t;
# else // __unix && USE_MMAP
#  undef USE_MMAP
/** File handle */
typedef FILE* file_t;
# endif // __unix && USE_MMAP

#endif // FILE_H_
