// Interface to the singly-linked slist template -*- c++ -*-

#ifndef SLIST
# define SLIST

# ifdef __GNUC__
#  if __GNUC__ < 3
#   include <slist>
#  else
#   include <ext/slist>
#   if __GNUC_MINOR__ == 0
#    define slist std::slist
#   else
#    define slist __gnu_cxx::slist
#   endif // __GNUC_MINOR__ == 0
#  endif // __GNUC__ < 3
# elif defined __sgi
#  include <slist>
#  define slist std::slist
# else // fall back to <list>
# include <list>

template <class T>
class slist
{
public:
  typedef T value_type;
  typedef value_type* pointer;
  typedef const value_type* const_pointer;
  typedef value_type& reference;
  typedef const value_type& const_reference;
  typedef size_t size_type;

private:
  typedef std::list<value_type> List;
  List myList;
public:
  slist () : myList () {}
  ~slist () {}

  size_type size () const { return myList.size (); }
  bool empty () const { return myList.empty (); }

  typedef typename List::iterator iterator;
  typedef typename List::const_iterator const_iterator;

  iterator begin () { return myList.begin (); }
  const_iterator begin () const { return myList.begin (); }
  iterator end () { return myList.end (); }
  const_iterator end () const { return myList.end (); }

  iterator insert (iterator pos, const value_type& x) {
    return myList.insert (pos, x);
  }

  iterator insert_after (iterator pos, const value_type& x) {
    if (pos != end ()) pos++;
    return myList.insert (pos, x);
  }

  iterator erase (iterator pos) {
    return myList.erase (pos);
  }

  iterator erase_after (iterator pos) {
    if (pos != end ()) pos++;
    return myList.erase (pos);
  }

  const value_type& front () const { return myList.front (); }
  value_type& front () { return myList.front (); }
  void pop_front () { myList.pop_front (); }
  void push_front (const value_type& x) { myList.push_front (x); }

  void clear () { myList.clear (); }
};
# endif
#endif // SLIST
